"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const core_2 = require("@collectable/core");
const set_1 = require("./set");
const unwrap_1 = require("./unwrap");
const internals_1 = require("../internals");
function fromNumericKeys(keys, mutability) {
    return fromKeys(core_2.numericCompare, keys, mutability);
}
exports.fromNumericKeys = fromNumericKeys;
function fromStringKeys(keys, mutability) {
    return fromKeys(core_2.stringCompare, keys, mutability);
}
exports.fromStringKeys = fromStringKeys;
function fromKeys(compare, keys, mutability) {
    const tree = core_1.Mutation.modify(internals_1.createTree(compare, mutability));
    if (Array.isArray(keys)) {
        for (var i = 0; i < keys.length; i++) {
            set_1.set(keys[i], null, tree);
        }
    }
    else {
        const it = internals_1.isRedBlackTree(keys) ? unwrap_1.keys(keys) : keys[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            set_1.set(current.value, null, tree);
        }
    }
    return core_1.Mutation.commit(tree);
}
exports.fromKeys = fromKeys;
function fromPairsWithNumericKeys(pairs, mutability) {
    return fromPairs(core_2.numericCompare, pairs, mutability);
}
exports.fromPairsWithNumericKeys = fromPairsWithNumericKeys;
function fromPairsWithStringKeys(pairs, mutability) {
    return fromPairs(core_2.stringCompare, pairs, mutability);
}
exports.fromPairsWithStringKeys = fromPairsWithStringKeys;
/**
 * Creates a new `RedBlackTree` from an array of key/value pairs (tuples). If no ComparatorFn function is supplied, keys
 * are compared using logical less-than and greater-than operations, which will generally only be suitable for numeric
 * or string keys.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {[K, V][]} pairs An array of pairs (tuples), each being a two-element array of [key, value]
 * @param {ComparatorFn<K>} compare A comparison function, taking two keys, and returning a value less than 0 if the
 *                                  first key is smaller than the second, a value greater than 0 if the first key is
 *                                  greater than the second, or 0 if they're the same.
 * @returns {RedBlackTreeStructure<K, V>} A tree populated with an entry for each pair in the input array
 */
function fromPairs(compare, pairs, mutability) {
    const tree = core_1.Mutation.modify(internals_1.createTree(compare, mutability));
    var pair;
    if (Array.isArray(pairs)) {
        for (var i = 0; i < pairs.length; i++) {
            pair = pairs[i];
            set_1.set(pair[0], pair[1], tree);
        }
    }
    else {
        const it = pairs[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            pair = current.value;
            set_1.set(pair[0], pair[1], tree);
        }
    }
    return core_1.Mutation.commit(tree);
}
exports.fromPairs = fromPairs;
function fromObject(obj, compare, mutability) {
    if (core_1.Mutation.isMutationContext(compare)) {
        mutability = compare;
        compare = void 0;
    }
    if (core_2.isUndefined(compare)) {
        compare = core_2.stringCompare;
    }
    const tree = core_1.Mutation.modify(internals_1.createTree(compare, mutability));
    const keys = Object.keys(obj);
    for (var i = 0; i < keys.length; i++) {
        var key = keys[i];
        set_1.set(key, obj[key], tree);
    }
    return core_1.Mutation.commit(tree);
}
exports.fromObject = fromObject;

//# sourceMappingURL=from.js.map
